// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ParallelTypes.h */

#include <vector>
#include <set>
#include <map>
#include "Global.h"
#include "parallel/InteriorBoundary.h"

#ifndef AMDIS_PARALLEL_TYPES_H
#define AMDIS_PARALLEL_TYPES_H

namespace AMDiS {

  using namespace std;

  /// Defines set of DOF indices.
  typedef std::set<DegreeOfFreedom> DofIndexSet;

  /// Defines a mapping type from DOFs to rank numbers.
  typedef map<const DegreeOfFreedom*, int> DofToRank;
  
  /// Defines a mapping type from DOFs to a set of rank numbers.
  typedef map<const DegreeOfFreedom*, std::set<int> > DofToPartitions;

  /// Defines a mapping type from DOF indices to a set of rank numbers.
  typedef map<DegreeOfFreedom, std::set<int> > DofIndexToPartitions;
  
  /// Defines a mapping type from rank numbers to sets of DOFs.
  typedef map<int, DofContainer> RankToDofContainer;
  
  /// Defines a mapping type from DOF indices to DOF indices.
  //typedef map<DegreeOfFreedom, DegreeOfFreedom> DofMap;
  
  /// Defines a mapping type from DOFs to boolean values.
  typedef map<const DegreeOfFreedom*, bool> DofToBool;
  
  /// Defines a mapping type from DOF indices to boolean values.
  typedef map<DegreeOfFreedom, bool> DofIndexToBool;
  
  /// Forward type (it maps rank numbers to the interior boundary objects).
  typedef InteriorBoundary::RankToBoundMap RankToBoundMap;
  
  typedef map<const DegreeOfFreedom*, DegreeOfFreedom> DofIndexMap;

  /// Maps a boundary type, i.e., a boundary identifier index, to a periodic 
  /// DOF mapping. 
  typedef std::map<BoundaryType, map<DegreeOfFreedom, DegreeOfFreedom> > PeriodicDofMap;
  
  /// Different FE spaces may have different DOFs on the same mesh. Thus we
  /// need to have a periodic DOF mapping for each FE space.
  typedef std::map<const FiniteElemSpace*, PeriodicDofMap> PeriodicDofMapFeSpace;
  
  /// Is used if a DOF index is mapped to multiple indices, i.e., to both, a local
  /// and a global one.
  struct MultiIndex
  {
    int local, global;
  };

  typedef std::map<DegreeOfFreedom, MultiIndex> DofMap;
 
  typedef vector<MeshStructure> MeshCodeVec;
}

#endif
